//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "header.h"
#include "mainapp.h"
#include "mpu680x0.h"
#include "pluto.h"
#include "rtc.h"
#include "vm.h"

void cli_halt_callback(void *);

// wxApp と相似にするためのクラス
class CLIApp
{
 public:
	int OnInit();
	bool Parse();
	[[noreturn]] void OnCreate();

	int ac;
	char **av;
};

int
main(int ac, char *av[])
{
	std::unique_ptr<CLIApp> app(new CLIApp());

	// wxApp とよく似せるため
	app->ac = ac;
	app->av = av;
	return app->OnInit();
}

// ここがエントリポイント(のつもり)
int
CLIApp::OnInit()
{
	int rv;

	rv = gMainApp.Init(true, ac, av);
	if (rv != MainApp::PASS) {
		return rv;
	}
	if (!gMainApp.Start()) {
		return EXIT_FAILURE;
	}

	// メインウィンドウに相当するところ
	OnCreate();

	return EXIT_SUCCESS;
}

// メインウィンドウ作成後のイベントに相当するところ
void
CLIApp::OnCreate()
{
	// VM にコールバックを登録
	if (gMPU680x0) {
		gMPU680x0->SetHaltCallback(cli_halt_callback, NULL);
	}

	// 時刻開始?
	// なるべく VM が開始する直前にしたい。
	gRTC->InitRTC();

	// VM 実行開始
	gVM->PowerButton();

	// ?
	for (;;) {
		sleep(1);
	}
}

// m680x0 ダブルバスフォールト時に呼ばれるコールバック。
void
cli_halt_callback(void *dummy)
{
	// どうする?
	warnx("Double bus fault has occurred in VM");
}
