<!--
 nono
 Copyright (C) 2020 nono project
 Licensed under nono-license.txt
-->
<html><?php
	$nono_ver = "";

	if ($_SERVER['argc'] > 1)
		get_version($_SERVER['argv'][1]);

function get_version($filename)
{
	global $nono_ver;

	$fp = fopen($filename, "r");
	if ($fp !== false) {
		while (($buf = fgets($fp)) !== false) {
			if (preg_match("/NONO_MAJOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_major = $m[1];
			}
			if (preg_match("/NONO_MINOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_minor = $m[1];
			}
			if (preg_match("/NONO_PATCH_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_patch = $m[1];
			}
			if (preg_match("/NONO_DATE\s+\"([^\"]+)\"/", $buf, $m)) {
				$nono_date = $m[1];
			}
		}
		fclose($fp);

		$nono_ver = "${nono_major}.${nono_minor}.${nono_patch} (${nono_date})";
	}
}
?>
<?php require_once "common.php"; ?>
<body>
<h3>nono <?=$nono_ver?></h3>

nono は NetBSD とかで動作する OMRON LUNA-I とかのエミュレータです。
<q>nono is OMRON LUNA-I emulator runs on NetBSD and etc.</q>
<hr>

<h4>はじめに <q>Introduction</q></h4>
<div class="main">
nono は NetBSD とかで動作する OMRON LUNA-I とかのエミュレータです。
LUNA88K は実験的サポートです、まだ動きません。
<q>
nono is OMRON LUNA-I emulator runs on NetBSD and etc.
LUNA88K is still experimental support and it does not work yet.
</q>
</div>

<h4>ビルド方法 <q>How to build</q></h4>
<div class="main">
ビルドには以下が必要です。
NetBSD(pkgsrc) なら wxWidgets は pkgsrc/x11/wxGTK30 (OPTIONS:gtk2 推奨) です。
<q>The followings are required for build.
If you use NetBSD(pkgsrc),
wxWidgets is pkgsrc/x11/wxGTK30 (OPTIONS:gtk2 is recommended).</q>

<ul>
<li>make (BSD make, not GNU make)
<li>C/C++ compiler which supports -std=c++14.
<br>
(For gcc, 7.4 works but 5.5 doesn't work, at least.
For clang, 7.0 and 8.0 works at least.)
<li>wxWidgets &gt;= 3.0
<li>gettext
</ul>

<p>
(NetBSD 以外でのビルドはサポートしていませんが)
Ubuntu 18.04 ではたぶん以下のパッケージが必要です。
<q>You may need the following packages on Ubuntu 18.04
(though we won't support non-NetBSD platform).</q>
</p>
<dl>
<dd>
bmake
build-essential
gettext
libbsd-dev
libedit-dev
libkqueue-dev
libwxgtk3.0-dev
zlib1g-dev
</dl>

<p>
アーカイブを展開したら以下のようにビルドします。
configure には環境変数 CC、CXX でコンパイラを指定することが出来ます。
また wx-config が標準的な名前で提供されていないために見付けられない場合には
環境変数 WX_CONFIG にパスを指定することが出来ます。
<q>Extract the archive and build as following.
You can specify C/C++ compiler using environment variable CC and CXX
if configure cannot find standard name suitable compiler.
Also, you can specify wx-config path using environment variable WX_CONFIG
if configure cannot find wx-config.</q>
<blockquote class="cons"><pre>
% ./configure
% make depend
% make
% su
# make install
</pre></blockquote>

<p>
2つの実行ファイルがインストールされます。
<tt>nono</tt> が GUI 版実行ファイル、
<tt>nono-cli</tt> がコマンドライン版です。
<q>Two executables will be installed.
<tt>nono</tt> is GUI executable and <tt>nono-cli</tt> is
command line executable.</q>

</div>


<h4>コマンドラインオプション <q>Command Line Option</q></h4>
<div class="main">
<dl>
<?php
function item($name, $desc)
{
	if (!is_array($name)) {
		$name = array($name);
	}
	foreach ($name as $n) {
		print "<dt class=dt-indent><tt>{$n}</tt></dt>\n";
	}
	print "<dd>{$desc}</p></dd>\n";
}
function option_item($name, $desc)
{
	item($name, $desc);
}

option_item("-A <i>file</i>", <<<_EOM_
ホストの <tt><i>file</i></tt> をロードして実行します。
<tt><i>file</i></tt> が相対パスの場合カレントディレクトリからのパスになります。
ファイルが gzip 圧縮されていれば自動的に展開します。
(展開後の) ファイル形式は a.out (OMAGIC) か ELF で、
実際にはブートローダとカーネル程度しか想定していません。
設定ファイルの <tt>prom-image</tt> とともに指定されると
<tt>-A</tt> のほうが優先します。
<q>Loads and executes host's <tt><i>file</i></tt>.
If <tt><i>file</i></tt> is relative path,
it is path from the current directory.
If the file is gzip'd, it is automatically extracted.
The supported file format (after extracting) is a.out (OMAGIC) or ELF.
Actually, it only assumes bootloaders or kernels.
If this option is specified at the same time as
<tt>prom-image</tt> in configuration file,
this option preceeds.
</q>
_EOM_
);

option_item("-c <i>vmdir</i>", <<<_EOM_
VM ディレクトリを指定します。省略するとカレントディレクトリとします。
<q>Specifies the VM directory.  If omitted, the current directory is used.</q>
_EOM_
);

option_item("-f", <<<_EOM_
高速モードで起動します。
GUI なら起動後にもメニューから変更できますが、その初期値を変えるだけです。
<q>Boot as the fast mode.
You can change this mode on GUI menu after boot,
and the option only changes its initial state.
</q>
_EOM_
);

option_item("--fontsize <i>height</i>", <<<_EOM_
GUI 版のみ。
全サブウインドウの起動時のフォントサイズを指定します。
<tt><i>height</i></tt> には 12, 16
のいずれかを指定します。デフォルトは <tt>12</tt> です。
起動後にメニューから変更できます。
<q>
GUI Only.
Specifies the initial fontsize on all sub windows.
<tt><i>height</i></tt> is one of 12 or 16.
The default value is <tt>12</tt>.
You can change this value on GUI menu after boot.
</q>
_EOM_
);

option_item(array("-s <i>scale</i>",
	"--scale <i>scale</i>"), <<<_EOM_
GUI 版のみ。
メインウィンドウの起動時のスケールを実数で指定します。
起動後にもメニューからプリセットされた倍率には変更可能ですが、
任意倍率は起動時のみ指定可能です
<span class=strike>(そのうちなんとかしたい)</span>。
デフォルトは 1.0 です。
<q>
GUI Only.
Specifies the initial main window scale in real number.
You can change this scale on GUI menu after boot,
but unlike this option, there are only a few preset choices
<span class=strike>(Should be improved in someday)</span>.
The default value is 1.0</q>
_EOM_
);

option_item("--show-config", <<<_EOM_
設定ファイルと <tt>-V</tt> オプションを読み込んだ結果を表示します。
<q>Shows the result of reading configuration file and
parsing <tt>-V</tt> options.</q>
_EOM_
);

option_item("-v", <<<_EOM_
バージョンを表示します。
<q>Shows the version.</q>
_EOM_
);

option_item("-V <i>name</i>=<i>value</i>", <<<_EOM_
設定ファイルで指定した <tt><i>name</i>=<i>configvalue</i></tt> の代わりに
このオプションの <tt><i>name</i>=<i>value</i></tt> を適用します。
<q>Use this <i>name</i>=<i>value</i>
instead of <i>name</i>=<i>configvalue</i> specified in configuration file.
</q>
_EOM_
);
?>
</dl>

以下開発用。<q>For developers:</q>
<dl>
<?php
// 項目間を空けない
function option_dev($name, $desc)
{
	print "<dt class=dt-indent><tt>{$name}</tt></dt>\n";
	print "<dd>{$desc}</dd>\n";
}

option_dev("-b <i>hexaddr</i>[,<i>skipcount</i>]", <<<_EOM_
デバッガのブレークポイントを 16進数で指定します。
_EOM_
);

option_dev("-C", <<<_EOM_
ログをコンソールにも出力します。
通常はログウィンドウにだけ出力されます。
_EOM_
);

option_dev("-d", <<<_EOM_
起動時にデバッガプロンプトで停止します。
_EOM_
);

option_dev("-D", <<<_EOM_
コンソールをデバッガとして使用します。
-d を指定しなくても起動時にプロンプトで停止します
<span class=strike>(そのうちなんとかしたい)</span>。
_EOM_
);

option_dev("-L <i>name1</i>=<i>level1</i>[,<i>name2</i>=<i>level2</i>,...]",
<<<_EOM_
ログレベルを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Lhelp</tt> で name の一覧を表示します。
_EOM_
);

option_dev("-M <i>name</i>[,<i>name2</i>,...]", <<<_EOM_
起動時に表示するモニタウィンドウを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Mhelp</tt> で name の一覧を表示します。
_EOM_
);


?>
</dl>
</div>

<h4>設定 <q>Configuration</q></h4>
<div class="main">
VM の設定ファイルはその VM ディレクトリ内の nono.cfg です。
書式は <tt>key = value</tt> 形式で1行1項目ずつです。
<tt>key</tt> と <tt>value</tt> の前後の空白は取り除かれます。
また空行と "<tt>#</tt>" で始まる行は無視します。
知らないキーの行も無視します。
<q>VM configuration file is nono.cfg in the VM directory.
Its syntax is <tt>key = value</tt> format, one per line.
White spaces before and after <tt>key</tt> and <tt>value</tt> are ignored.
And, blank lines, lines beginning with "<tt>#</tt>", and
lines with unrecognized key are also ignored.
</q>
<p>
設定項目は次の通りです。
<q>The configuration items are:</q>
<dl>
<?php
function config_item($name, $desc)
{
	item($name, $desc);
}

config_item("vmtype = <i>string</i>", <<<_EOM_
VM 種別を以下のいずれかから指定します。
省略不可です。
<q>Specifies the VM type from the following.
This field is mandatory.
</q>
<table cellspacing=0 cellpadding=0>
<tr><td>&nbsp;<td><tt>luna</tt>	<td>… LUNA-I
<tr><td><td><tt>luna88k</tt>	<td>… LUNA88K (experimental)
</table>
_EOM_
);

config_item("debugger-port = <i>integer</i>", <<<_EOM_
デバッガの TCP 待ち受けポート番号を指定します。
0 なら待ち受けを行いません。
デフォルトは 0 です。
<q>Specifies the TCP port number that debugger listens.
If 0, it will not listen.
The default is 0.</q>
_EOM_
);

config_item("ethernet-hostdriver = <i>string</i>", <<<_EOM_
イーサネットデバイスのホスト側ドライバを指定します。
<tt>none</tt>、<tt>tap</tt>、<tt>bpf</tt> が指定できます。
<tt>none</tt> ならホスト側とは一切通信を行いません。
<tt>tap</tt> か <tt>bpf</tt> かはホスト OS によってビルド時に決定します。
NetBSD なら <tt>tap</tt> です。
_EOM_
);

config_item("ethernet-macaddr = <i>string</i>", <<<_EOM_
イーサネットデバイスの仮想マシン側の MAC アドレスを指定します。
<tt><i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i></tt>
形式で指定します。
<tt>auto</tt> なら自動的に決定します。
デフォルトは <tt>auto</tt> です。
_EOM_
);

config_item("luna-dipsw1 = <i>string</i>", <<<_EOM_
本体前面 DIPSW#1-1..#1-8 の内容を指定します。
"<tt>0</tt>" を DOWN、"<tt>1</tt>" を UP として、
これを8つ並べた形式で、前から順に #1..#8 に対応します。
<q>Specifies status of the front panel DIPSW#1-1..#1-8 using 8 digit.
"<tt>0</tt>" means DOWN and "<tt>1</tt>" means UP.
The first character corresponds to #1 and
the eighth character corresponds to #8.</q>
<p>
LUNA-I でのデフォルトは <tt>11110111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA-I, the default value is <tt>11110111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="https://wiki.netbsd.org/ports/luna68k/luna68k_info/"
>NetBSD/luna68k: Information</a>
<br>
LUNA88K でのデフォルトは <tt>11111111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA88K, the default value is <tt>11111111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="http://man.openbsd.org/boot_luna88k.8"
>OpenBSD manual pages: boot_luna88k(8)</a>
_EOM_
);

config_item("luna-dipsw2 = <i>string</i>", <<<_EOM_
本体前面 DIPSW#2-1..#2-8 の内容を指定します。
書式は <tt>luna-dipsw1</tt> と同じです。
デフォルトは <tt>11111111</tt> です。
<q>Specifies status of the front panel DIPSW#2-1..#2-8.
The same syntax as <tt>luna-dipsw1</tt> is used.
The default value is <tt>11111111</tt>.</q>
<p>
NetBSD/luna68k のブートローダは、
DIPSW#2 が "<tt>11111111</tt>" なら自動的にカーネルをロードして実行し、
どれかでも "<tt>0</tt>" にするとプロンプトで停止するようです。
<span class=strike>(本当は #8 だけで制御するつもりだったんじゃないかという気がします)</span>
<q>NetBSD/luna68k bootloader will automatically load and execute the kernel,
if the DIPSW#2 is "<tt>11111111</tt>".
Otherwise, the bootloader will enter interactive mode.
<span class=strike>(I doubt that they actually wanted to switch with only #8)
</span>
</q>
_EOM_
);

config_item("monitor-rate = <i>integer</i>", <<<_EOM_
テキスト系モニタウィンドウの更新頻度を Hz 単位で指定します。
1 から 60 までの間で指定でき、デフォルトは 20Hz です。
起動後にメニューからプリセットされた頻度には変更可能です。
<q>Specifies refresh rate of all text monitor windows in Hz.
It ranges from 1 to 60.  The default is 20Hz.
You can change this value on GUI menu after boot,
but unlike this configuration value, there are only a few preset choices.
</q>
_EOM_
);

config_item("mpu-clock = <i>value</i>", <<<_EOM_
MPU のクロック数を MHz 単位で指定します。
デフォルトは LUNA-I なら 20MHz、LUNA88K なら 25MHz です。
<q>Specifies the MPU clock in MHz.
The default value is 20MHz on LUNA-I, or 25MHz on LUNA88K.</q>
_EOM_
);

/* luna には不要
  config_item("mpu-has-fpu", ""); */

config_item("prom-image = <i>path</i>", <<<_EOM_
LUNA-I/LUNA88K の外部 ROM イメージファイルのパスを指定します。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると内蔵 ROM を使用します。
デフォルトは空です。
<q>Specifies the LUNA-I/LUNA88K's external ROM image file path.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current
directory.
If the <i>path</i> is empty, internal emulated ROM will be used.
The default value is empty.</q>
<p>
実機を持っていない場合はこの値を空に (= デフォルトのままに) しておくと、
nono 内蔵のなんちゃって下位互換 ROM で起動します(現状 LUNA-I のみ)。
<q>If you does not have the real LUNA machines,
you can boot with nono's internal downward compatible emulated ROM
if you set this field empty (or leave it as the default).
For now, it's only for LUNA-I.
</q>
<p>
LUNA-I 実機を持っている場合は
ROM ファイルを指定することで実機 ROM で起動できます。
ROM ファイルは実機の 0x41000000-0x4101ffff (128KB) を保存したものです。
今のところ ROM は V4.22 (Thu Jul 27 11:45:42 1989) のみサポートしています。
それ以外については何も分かりません。
<q>If you have the real LUNA-I machine,
you can boot with the real ROM spcifying the ROM file path.
The ROM file is extracted from 0x41000000-0x4101ffff (128KB) of
the real LUNA-I machine.
For now, only V4.22 (Thu Jul 27 11:45:42 1989) is supported.
I have no idea about other ROMs.
</q>
<p>
LUNA88K 実機の場合は 0x41000000-0x4103ffff (256KB) を保存したものです。
ただし現状サポートしていません。
<q>
For LUNA88K,
the ROM file is extracted from 0x41000000-0x4103ffff (256KB).
However, it's not supported yet.
</q>
_EOM_
);

config_item("ram-size = <i>integer</i>", <<<_EOM_
搭載する RAM サイズを MB 単位で指定します。
デフォルトは LUNA-I は 16MB、LUNA88K は 64MB です。
まだたぶん変更できません。
<q>Specifies the RAM size in MB.
The default is 16MB on LUNA-I or 64MB on LUNA88K.
Not supported yet.</q>
_EOM_
);

config_item("spc0-id<i>N</i>-image = <i>devtype</i>,<i>path</i>", <<<_EOM_
SCSI デバイスを指定します。<i>N</i> には 0 から 7 が入ります。
ID 7 は本体が使用しますので指定しないでください。
値はデバイス種別 <i>devtype</i> とイメージパス <i>path</i>
を "<tt>,</tt>"(カンマ) で区切って並べた形式で、
今の所デバイス種別 <i>devtype</i> には "<tt>hd</tt>"(ハードディスク) のみ指定可能です。
ディスクイメージパスが相対パスなら VM ディレクトリからの相対パスになります。
<q>
Specifies SCSI device.  <i>N</i> is 0 to 7.
But don't specify ID 7 because the host uses it.
The value is in a form of device type <i>devtype</i> and
the image path <i>path</i> separated by "<tt>,</tt>"(comma).
For now, only "<tt>hd</tt>" (hard disk) can be specified for <i>devtype</i>.
If the <i>path</i> is relative path, it is from the VM directory.
</q>

<p>
例えば、nono.cfg と同じディレクトリに置いたディスクイメージ luna.img
を使う場合、
LUNA-I では通常 ID 6 をプライマリ HDD に割り当てるのでこんな感じになります。
<q>For example, if you use disk image luna.img placed in the same
directory as nono.cfg,
since LUNA-I usually assigns ID 6 to the primary HDD,
write as following:</q>
<blockquote><pre>
spc0-id6-image = hd,luna.img
</pre></blockquote>
_EOM_
);

config_item("spc0-id<i>N</i>-seektime = <i>integer</i>", <<<_EOM_
指定の SCSI HDD の平均シークタイムを msec 単位で指定します。
現在のデフォルトは <tt>0</tt> です (S・S・D!! S・S・D!!)。
16 程度を指定すると幾分往時に思いを馳せることが出来るかもしれませんが、
今の所あまり安定していません。
<q>Specifies the average seek time of specified SCSI HDD in msec.
Currently, the default value is <tt>0</tt>
(This may be something like SSD :-).
If you specify about 16 or so, you can feel nostalgic,
but this feature is still unstable.
</q>
_EOM_
);

config_item("spc0-id<i>N</i>-writeprotect = <i>integer</i>", <<<_EOM_
指定の SCSI デバイスへの書き込みを無視するかどうか指定します。
<tt>0</tt> なら通常動作(書き込みを行う)です。
<tt>1</tt> なら書き込みコマンドは成功したように振る舞いますが実際には
ディスクイメージに一切書き戻しません。
fsck を気にせずカーネルのデバッグとかを行いたい場合にはどうぞ。
何が起きるか意味が分からない人は指定しないでください。
デフォルトは <tt>0</tt> です。
<q>Specifies whether nono ignores writing to SCSI devices.
<tt>0</tt> means normal operation (writes to the devices).
If <tt>1</tt> is specified, 
nono will not actually write back to the disk image
even though the write command is succeeded.
nono's SCSI devices acts as
write command is successfully done but it never writes back
to the actual disk image.
This is useful for kernel debugging because it does not require fsck
after the kernel hangs.
But don't use this flag if you don't understand this paragraph.
The default value is <tt>0</tt>.
</q>
_EOM_
);


?>
</dl>
</div>


<h4>実行してみる <q>Try it</q></h4>
<div class="main">
つついさんが NetBSD/luna68k 9.0 の liveimage を用意してくださっています
(いつもありがとうございます)。
<q>Tsutsui-san has provided NetBSD/luna68k 9.0 liveimage for this
(Thanks as always).</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/luna/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectory for individual VMs in it (for example ~/nono/luna).</q>

<li>以下のリンクからイメージファイルをダウンロードして展開し、
VM ディレクトリ ~/nono/luna/ に置きます。
<q>Download imagefile from the following link,
extract it and place it in the VM directory, ~/nono/luna.</q>
<blockquote>
<a href="http://teokurebsd.org/netbsd/liveimage/20200518-luna68k/"
>http://teokurebsd.org/netbsd/liveimage/20200518-luna68k/</a>
</blockquote>

<li>同じく VM ディレクトリに設定ファイル nono.cfg を以下の内容で用意します。
<q>Create a configuration file nono.cfg in the same VM directory, ~/nono/luna,
with following contents:</q>
<blockquote class="file"><pre>
vmtype = luna
spc0-id6-image = hd,liveimage-luna68k-raw-20200518.img
</pre></blockquote>
<li><tt>nono -c ~/nono/luna</tt> で起動します
(VM ディレクトリに自動的に NVRAM.DAT が作られます)。
<q>Run as <tt>nono -c ~/nono/luna</tt>.
(NVRAM.DAT will be created automatically in the VM directory)</q>
<li>Emulated ROM Monitor が起動するので、
初回は以下のように入力すると NetBSD が起動します。
<q>The emulated ROM Monitor will be executed.
Then, only for the first time,
entering the following can boot NetBSD.</q>
<blockquote class="file"><pre>
k
[Enter]
[Enter]
d
boot
g
x
</pre></blockquote>
この内容は NVRAM.DAT に記録されているので次回以降は直接 NetBSD が起動します。
<q>The information you have just entered is recorded in NVRAM,
so next time it boots NetBSD automatically.</q>
</ol>
<p>
<a href="https://twitter.com/tsutsuii/status/1262429647364427783"
>元ツイートはこちら<q>Original tweet</q></a>。
</div>


<h4>ネットワーク設定 <q>Setup network</q></h4>
<div class=main>
設定ファイル nono.cfg に以下の行を追加します。
<q>Add the following line to configuration file, nono.cfg.</q>
<blockquote class="file"><pre>
ethernet-hostdriver = tap
</pre></blockquote>
次に VM ディレクトリかその親ディレクトリに nono-ifup、nono-ifdown
というスクリプトを用意します。
nono は tap(4) をオープンし、
そのデバイス名を引数にこれらのスクリプトを呼びます。
<q>Then, prepare two scripts named nono-ifup and nono-ifdown
in the VM directory or its parent directory.
nono will open tap(4) and
invoke these scripts with the name of the device as an argument.</q>
<p>
例えば、NetBSD ホストで、ホストの wm0 セグメントに nono のゲスト OS を接続する場合、
<q>For example, if you want to connect guest OS to host's wm0 segment
on NetBSD,</q>
<ol>
<li>デフォルトでは /dev/tap は一般ユーザからアクセスできないので、
chmod で適当にパーミッションを与えます。
番号の付いていないほうの /dev/tap だけでいいです。
<q>By default, /dev/tap is only accessible to privileged user.
You need to chmod /dev/tap (without unit number) appropriately.</q>
sysinst 等で OS をアップグレードするとパーミッションが 600
に戻るのがハマりポイントです。

<li>bridge(4) インタフェースを作成し、
ホストの外側のインタフェースをブリッジに追加しておきます。
<q>Create a bridge(4) interface, and
add your physical interface to the bridge.</q>
<blockquote class="cons"><pre>
# ifconfig bridge0 create
# brconfig bridge0 add wm0
</pre></blockquote>
常用するなら設定ファイルに書いておきましょう。
<q>If you want to use this all the time,
you can put configuration file into /etc.</q>
<blockquote>
/etc/ifconfig.bridge0
<pre class=file>
create
up
!/sbin/brconfig $int add wm0
</pre>
<pre class=cons>
# /etc/rc.d/network restart
</pre>
</blockquote>

<li>
一般ユーザに戻って、
VM ディレクトリかその親ディレクトリに
次のような 2つのスクリプトを用意します。
sudo の設定は別途行ってください。
<q>Return to non-privileged user, and
create following two scripts in the VM directory or its parent directory.
In addition, you need to set up sudo separately.</q>
<blockquote>
nono-ifup
<pre class=file>
#!/bin/sh
sudo /sbin/ifconfig $1 up
sudo /sbin/brconfig bridge0 add $1
</pre></blockquote>

<blockquote>
nono-ifdown
<pre class=file>
#!/bin/sh
sudo /sbin/brconfig bridge0 delete $1
sudo /sbin/ifconfig $1 down
</pre></blockquote>

<blockquote class="cons"><pre>
% chmod +x nono-ifup nono-ifdown
</pre></blockquote>

<li>nono を起動し、
メニューの「モニタ &gt; ホスト &gt; ホストネットワーク」を開いて
NetDriver が None 以外 (この場合 BSD tap) になっていれば動いてるはずです。
<q>Run nono,
and open "Monitor &gt; Host &gt; Host Network" window from menu.
It's OK if you can see "NetDriver: BSD tap" (in this case).</q>

</ol>

</div>


<h4>変更履歴 <q>Changes</q></h4>
<div class="main">
See <a href="changes.html">changes.html</a>.
</div>


<h4>ライセンス <q>License</q></h4>
<div class="main">
See <a href="nono-license.txt">nono-license.txt</a>.
</div>


<h4>連絡先 <q>Contact us</q></h4>
<div class="main">
バグ報告などは以下にお願いします。日本語でおk。
<q>If you find any problems, please let me know.
You may write in English.</q><br>
<a href="https://github.com/isaki68k/nono-issue/issues"
>https://github.com/isaki68k/nono-issue/issues</a>
</div>


<h4>パッチの提供について <q>About contributes</q></h4>
<div class="main">
パッチを提供してくださる場合は以下に同意したものとします。
<q>If you provide a patch to nono, you must agree to the following conditions:
</q>
<ul>
<li>成果物が nono のライセンスに従って運用あるいは配布されること。
<q>All your work are operated or distributed under the nono license.</q>
<li>ライセンスが将来変わる可能性があること。
<q>The license may be changed in the future.</q>
<li>著作部分に関して著作者人格権を行使しないこと。
<q>Do not exercise your author's rights.</q>
</ul>
<ul>
</div>


<h4>Acknowledgements</h4>
<div class="main">
nono は以下の広告条項を含むソースコードを利用しています。
<q>nono uses source code with the following advertising clause.</q>
<blockquote>
This product includes software developed by Gordon Ross<br>
This product includes software developed by the University of California, Lawrence Berkeley Laboratory.<br>
</blockquote>
</div>


<hr>
nono project
</body>
</html>
