//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "header.h"
#include <vector>

// 固定長キュー
// std::queue では出来ないことがいくつかあるので仕方なく車輪の再開発。
//
// T 型で上限 capacity 個のキュー。
// スレッドセーフではない。
template <typename T, int capacity>
class FixedQueue
{
 public:
	// コンストラクタ
	FixedQueue()
		: buf(capacity)
	{
		Clear();
	}

	// デストラクタ
	virtual ~FixedQueue() {
	}

	// 空にする
	void Clear() {
		start = 0;
		length = 0;
	}

	// 追加
	bool Enqueue(T val) {
		if (length < capacity) {
			// 空きがあれば追加
			buf[(start + length) % capacity] = val;
			length++;
			return true;
		} else {
			return false;
		}
	}

	// 取り出し
	// 要素が1つ以上あれば *outp に取り出して true を返す。
	// キューが空なら false を返す。
	bool Dequeue(T *outp) {
		if (length > 0) {
			// 1つ以上あれば取り出し
			*outp = buf[start];
			start = (start + 1) % capacity;
			length--;
			return true;
		} else {
			return false;
		}
	}

	// 取り出し
	// 先頭の要素を取り出して返す。
	// キューが空の時に呼ぶと不定値が返る。
	T Dequeue() {
		T val = buf[start];
		if (length > 0) {
			start = (start + 1) % capacity;
			length--;
		}
		return val;
	}

	// 現在の要素数を取得
	uint Length() const {
		return length;
	}

	// 要素が一杯なら true を返す
	bool IsFull() const {
		return (length >= capacity);
	}

	// 要素を覗き見る。
	// idx は 0 .. length-1 までで、0 がキューの先頭。
	T Peek(int idx) const {
		// XXX 範囲チェックすべきだがとりあえず
		return buf[(start + idx) % capacity];
	}

 private:
	std::vector<T> buf {};		// バッファ
	uint start {};				// 開始位置
	uint length {};				// 現在有効な長さ
};
