//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "config.h"
#include "debugger.h"
#include "logger.h"
#include "vm_luna.h"
#include "vm_x68k.h"

// 起動時の引数処理などを行う。
class MainApp
{
 public:
	// Init() の戻り値
	static const int PASS = -1;		// 実行を継続

 public:
	// 初期化
	int Init(bool is_cli_, int ac, char *av[]);

	// VM 実行開始
	bool Start();

	std::string SearchFile(const std::string& name) const;

	// vmtype (VMTYPE_*) を返す。
	// (Config が持っているのは文字列)
	vmtype_t GetVMType() const { return vmtype; }

	// 現在の VM が指定の feature を持っているか?
	bool HasVMFeature(vmf_t flag) const;

 public:
	// -A: ホストファイル名 (とりあえず)
	const char *host_file {};

	// -b: ブレークポイント "<addr>[,<skip>]"
	std::vector<std::string> debug_breakaddr {};

	// -B: ベンチマークモード
	int benchmark_mode {};

	// -c: VM ディレクトリ
	std::string vmdir {};
	const std::string& GetVMDir() const { return vmdir; }

	// -C: ログをコンソールに出力
	bool log_to_console {};

	// -d: 起動時にデバッガで停止
	bool debug_on_start {};

	// -D: コンソールをデバッガで使う
	bool debug_on_console {};

	// -f: 高速モード (XXX 設定ファイルと含めて見直すこと)
	bool fast_mode {};

	// --fontsize: フォントサイズ指定値 (値は fontsize_t ではない)
	int  fontsize {};

	// -L: ログ指定文字列
	char logopt[256] {};

	// -M: モニタ名
	std::string monitor_opt {};

	// -s,--scale: 画面スケール
	double screen_scale {};

	// --show-config: 設定内容を表示 (1なら通常、2ならall)
	int show_config {};

	// -V: パラメータ指定
	std::vector<std::string> config_options {};

	// -X: Human68k 実行ファイル名と引数
	const char *human68k_file {};
	char human68k_arg[256] {};

 private:
	// ヘルプメッセージを表示する
	void ShowHelp(bool all) const;

	// バージョンを表示する
	void ShowVersion() const;

	// 引数を処理する
	bool ParseOpt(int ac, char *av[]);

	// -L オプションを処理する
	bool ParseLogopt();

	// CLI/GUI 区別
	bool IsCLI() const { return  is_cli; }
	bool IsGUI() const { return !is_cli; }
	bool is_cli {};

	// VM 種別
	vmtype_t vmtype {};
};

// グローバルインスタンス
extern MainApp gMainApp;
