//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "m88100.h"

// コンストラクタ
m88kcpu::m88kcpu(uint32 reset_vector_)
{
	reset_vector = reset_vector_;

	cmmu[0].Ctor(this);
	cmmu[1].Ctor(this);
}

// デストラクタ
m88kcpu::~m88kcpu()
{
}

// PID の VERSION フィールドをセットする。初期化時に呼ぶ。
void
m88kcpu::SetVersion(uint32 version)
{
	pid &= ~PID_VER_MASK;
	pid |= version << 1;
}

/*static*/ const char * const m88100reg::sipname[3] = {
	"sxip", "snip", "sfip",
};

/*static*/ const char * const m88100reg::dmt_en_str[16] = {
	"----",
	"---B",
	"--B-",
	"--HH",
	"-B--",
	"-1-1", // not used normally
	"-11-", // not used normally
	"-111", // not used normally
	"B---",
	"1--1", // not used normally
	"1-1-", // not used normally
	"1-11", // not used normally
	"HH--",
	"11-1", // not used normally
	"111-", // not used normally
	"LLLL",
};

// 例外
/*static*/ const char * const m88kcpu::exception_names[] = {
			//   01234567890123456789012 <- 例外履歴欄の横幅
	/*  0 */	"Reset Exception",
	/*  1 */	"Interrupt Exception",
	/*  2 */	"Inst Access Exception",
	/*  3 */	"Data Access Exception",
	/*  4 */	"Misaligned Access Excep",
	/*  5 */	"Unimplemented Opcode",
	/*  6 */	"Priv. Violation Excep.",
	/*  7 */	"Bounds Check Violation",
	/*  8 */	"Illegal Integer Divide",
	/*  9 */	"Int Overflow Exception",
	/* 10 */	"Error Exception",
			//   01234567890123456789012
};

// 例外名を返す。
// Reserved なところは NULL を返す。
// XXX TODO 114 以降未実装
/*static*/ const char *
m88kcpu::GetExceptionName(int vector)
{
	if (0 <= vector && vector < countof(exception_names)) {
		return exception_names[vector];
	}

	// OpenBSD
	// XXX 本来は OpenBSD 稼働時に限定すべきだろうけど、そうする意味もない
	switch (vector) {
	 case 450:
		return "OpenBSD system call";
	 case 451:
		return "OpenBSD cache flush";
	 case 503:
		return "Division by zero in GCC";
	 case 504:
		return "OpenBSD stepbpt";
	 case 511:
		return "OpenBSD userbpt";
	 default:
		break;
	}

	return NULL;
}
