//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "gvram.h"
#include "mpu.h"
#include "videoctlr.h"

//
// GVRAM
//
// GVRAM はワード単位でホストバイトオーダ配置なので、
// バイトアクセス時は HB() マクロを使用のこと。
//

std::unique_ptr<GVRAMDevice> gGVRAM;

GVRAMDevice::GVRAMDevice()
{
	logname = "gvram";
	devname = "GVRAM";
	devaddr = 0xc00000;
	devlen  = 0x200000;	// XXX とりあえず

	mem.reset(new uint8[devlen]);
}

GVRAMDevice::~GVRAMDevice()
{
}

uint64
GVRAMDevice::Read8(uint32 addr)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - devaddr;
	return mem[HB(offset)];
}

uint64
GVRAMDevice::Read16(uint32 addr)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - devaddr;
	return *(uint16 *)&mem[offset];
}

uint64
GVRAMDevice::Write8(uint32 addr, uint32 data)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - devaddr;
	mem[HB(offset)] = data;
	return 0;
}

uint64
GVRAMDevice::Write16(uint32 addr, uint32 data)
{
	gMPU->AddCycle(9);	// Inside/Out p.135
	uint32 offset = addr - devaddr;
	*(uint16 *)&mem[offset] = data;
	return 0;
}

uint64
GVRAMDevice::Peek8(uint32 addr)
{
	return mem[HB(addr - devaddr)];
}
