//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU 共通部

#pragma once

#include "device.h"
#include "event.h"

class MPUDevice : public Device
{
	using inherited = Device;
 public:
	MPUDevice();
	virtual ~MPUDevice() override;

	bool Init() override;
	void ResetHard() override;

	// MPU を vtime [nsec] 分実行する
	virtual uint32 Run(uint32 vtime) = 0;

	// 現在の仮想時刻 [nsec] を取得する。
	virtual uint64 GetVirtTime() const = 0;

	// MPU の処理をこの命令で中断させる。
	// スケジューラのイベントリストに追加が発生した時に VM 側から
	// MPU へ処理の中断を指示する。
	virtual void Release() = 0;

	// 現在実行中の命令の PC を取得
	virtual uint32 GetPPC() const = 0;

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	// Laddr == Paddr が MMU オフなのか PA=VA で運用中なのかの区別は
	// ここでは付かない。
	virtual uint32 GetLaddr() const = 0;
	virtual uint32 GetPaddr() const = 0;

	// MPU クロック [kHz] を取得する。
	int GetClockSpeed() const { return clock_khz; }

	// アクセスウェイトを加算
	virtual void AddCycle(int32 cycle) = 0;

	// 割り込み発生を MPU に通知
	virtual void Interrupt(int level) = 0;

 protected:
	int clock_khz {};		// MPU クロック [kHz]

	// イベント (リセット例外、割り込み)
	Event event {};
};

extern std::unique_ptr<MPUDevice> gMPU;
