//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

// MPU (m88xx0)

#pragma once

#include "mpu.h"
#include "m88100.h"

class MPU88xx0Device : public MPUDevice
{
	using inherited = MPUDevice;
 public:
	MPU88xx0Device(uint32 reset_vector);
	virtual ~MPU88xx0Device() override;

	bool Init() override;

	void MonitorUpdate(TextScreen&) override;

	// MPU を vtime [nsec] 分実行する
	uint32 Run(uint32 vtime) override { return cpu->Run(vtime); }

	// 現在の仮想時刻 [nsec] を取得する
	uint64 GetVirtTime() const override { return cpu->GetVTime(); }

	// MPU の処理をこの命令で中断させる。
	void Release() override { cpu->Release(); }

	// 現在実行中の命令の PC を取得
	uint32 GetPPC() const override { return cpu->GetXIP(); }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override { return 0; }	// XXX NotImplemented
	uint32 GetPaddr() const override { return 0; }	// XXX NotImplemented

	// アクセスウェイトを加算
	void AddCycle(int32 cycle) override;

	// 割り込み発生を MPU に通知
	void Interrupt(int level) override;

	// CPU コアを取得 (主にデバッグ用途)
	m88kcpu *GetCPU() const { return cpu.get(); }

 private:
	// イベントコールバック
	void Callback(Event& ev);

	std::unique_ptr<m88kcpu> cpu {};
};

// ATC モニタ
class MPU88200ATC : public Object
{
 public:
	MPU88200ATC(m88200 *cmmu);
	~MPU88200ATC() override { }

	void MonitorUpdate(TextScreen&) override;

 private:
	m88200 *cmmu {};
};

extern std::unique_ptr<MPU88200ATC> gMPU88200ATC[2];

#define gMPU88xx0	(dynamic_cast<MPU88xx0Device *>(gMPU.get()))
