//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"

class RAMDevice
	: public IODevice
{
	using inherited = IODevice;
 public:
	RAMDevice();
	virtual ~RAMDevice() override;

	bool Init() override;
	void ResetHard() override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	// アクセスウェイトを設定
	void SetWait(uint32 wait);

	// ホストの filepath をロードする
	uint32 LoadFile(const char *filepath, uint32 mid);
	// file, filesize で示されるバッファをロードする
	uint32 LoadFile(const char *name, const uint8 *file, size_t filesize,
		uint32 mid);
 private:
	uint32 LoadGzFile(const char *filepath, const uint8 *file, size_t filesize,
		uint32 mid);
	uint32 Load_aout(const char *name, const uint8 *file, size_t filesize,
		uint32 mid);
	uint32 Load_elf32(const char *name, const uint8 *file, size_t filesize,
		uint32 mid);

	uint32 read_wait {};
	uint32 write_wait {};
};

extern std::unique_ptr<uint8[]> ram;
extern int ram_size;
extern std::unique_ptr<RAMDevice> gRAM;
