//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "sio.h"
#include "interrupt.h"

//          IODevice
//              |
//              v
//        upd7201Device  (uPD7201 と Z8530 の共通部分)
//              |
//    +---------+----------+
//    v                    v
// SIODevice           SCCDevice
// (LUNA, uPD7201)     (X68030, Z8530)

std::unique_ptr<SIODevice> gSIO;

SIODevice::SIODevice()
{
	logname = "sio";
	devname = "SIO";
	devaddr = 0x51000000;

	cr.chan[0].desc = "Serial Console";
	cr.chan[1].desc = "Keyboard";
}

SIODevice::~SIODevice()
{
}

// 本体リセットスイッチによるリセット
void
SIODevice::ResetHard()
{
	// チップリセット
	Reset();
}

uint64
SIODevice::Read(uint32 addr)
{
	uint32 data;

	switch (addr) {
	 case 0:
		data = ReadData(&cr.chan[0]);
		break;
	 case 1:
		data = ReadCtrl(&cr.chan[0]);
		break;
	 case 2:
		data = ReadData(&cr.chan[1]);
		break;
	 case 3:
		data = ReadCtrl(&cr.chan[1]);
		break;
	 default:
		__unreachable();
	}
	return data;
}

uint64
SIODevice::Write(uint32 addr, uint32 data)
{
	switch (addr) {
	 case 0:
		WriteData(&cr.chan[0], data);
		return 0;

	 case 1:
		WriteCtrl(&cr.chan[0], data);
		return 0;

	 case 2:
		WriteData(&cr.chan[1], data);
		return 0;

	 case 3:
		WriteCtrl(&cr.chan[1], data);
		return 0;
	}
	__unreachable();
}

uint64
SIODevice::Peek(uint32 addr)
{
	switch (addr) {
	 case 0:
		return PeekData(&cr.chan[0]);

	 case 1:
		return PeekCtrl(&cr.chan[0]);

	 case 2:
		return PeekData(&cr.chan[1]);

	 case 3:
		return PeekCtrl(&cr.chan[1]);
	}
	__unreachable();
}

// 割り込みを上げる
void
SIODevice::Interrupt()
{
	putlog(2, "Interrupt");
	gInterrupt->AssertINT(this);
}
