//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#include "sysclk.h"
#include "config.h"
#include "interrupt.h"

std::unique_ptr<SysClkDevice> gSysClk;

SysClkDevice::SysClkDevice()
{
	logname = "sysclk";
	devname = "SystemClock";
	devaddr = 0x63000000;

	event.dev = this;
	event.func = (DeviceCallback_t)&SysClkDevice::Callback;
	event.code = 0;
	event.SetName("System Clock");
}

SysClkDevice::~SysClkDevice()
{
}

bool
SysClkDevice::Init()
{
	// クロックは機種ごとに異なる
	const ConfigItem& item = gConfig->Find("!sysclk-period");
	event.time = item.AsInt();
	return true;
}

bool
SysClkDevice::PowerOn()
{
	// 電源オンですぐ起動
	event.Start();
	return true;
}

uint64
SysClkDevice::Read(uint32 addr)
{
	// 1 ポートしかない
	return GetInt();
}

uint64
SysClkDevice::Write(uint32 addr, uint32 data)
{
	// 何か書き込まれたら、フラグをクリア
	putlog(1, "システムクロック割り込みクリア");
	sysint = false;
	gInterrupt->NegateINT(this);
	return 0;
}

uint64
SysClkDevice::Peek(uint32 addr)
{
	return GetInt();
}

// 割り込み状態を取得(?)
uint8
SysClkDevice::GetInt() const
{
	uint8 data = 0;

	if (sysint) {
		data |= 0x80;
	}
	return data;
}

// イベントハンドラ
void
SysClkDevice::Callback(Event& ev)
{
	putlog(3, "システムクロック割り込み発生");
	sysint = true;
	gInterrupt->AssertINT(this);
	ev.Start();
}
