//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"

class VideoCtlrDevice
	: public IODevice
{
	using inherited = IODevice;
 private:
	static const int baseaddr = 0xe82000;

	struct videoctlr {
		uint16 r0;
		uint16 r1;
		uint16 r2;
	};

 public:
	VideoCtlrDevice();
	virtual ~VideoCtlrDevice() override;

	void ResetHard() override;

	// 加工済みテキストパレットの先頭アドレスを取得
	// レンダラとかからの参照用
	const uint32 *GetCookedPalette() const { return cooked_palette; }

 protected:
	// BusIO インタフェース
	static const uint NPORT = 4096;
	uint64 Read(uint32 addr);
	uint64 Write(uint32 addr, uint32 data);
	uint64 Peek(uint32 addr);

 private:
	void SetR0(uint32 data);
	void SetR1(uint32 data);
	void SetR2(uint32 data);
	void MakePalette();

	struct videoctlr videoctlr {};
	union {
		uint8 b[1024];
		uint16 w[512];
	} palette {};

	// xBGR32 形式のテキストパレット (レンダラ用に加工したもの)
	uint32 cooked_palette[16] {};
};

extern std::unique_ptr<VideoCtlrDevice> gVideoCtlr;
