//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "device.h"

// VM 種別
enum vmtype_t {
	VMTYPE_NONE = 0,
	VMTYPE_X68030,
	VMTYPE_LUNA1,
	VMTYPE_RXZ,		// Human68k .r .x .z console emulation
	VMTYPE_LUNA88K,
};
// VM 種別 (ビットマスク表現)
enum vmf_t {
	VMF_NONE	= 0,
	VMF_X68030	= (1 << VMTYPE_X68030),
	VMF_LUNA1	= (1 << VMTYPE_LUNA1),
	VMF_RXZ		= (1 << VMTYPE_RXZ),
	VMF_LUNA88K	= (1 << VMTYPE_LUNA88K),

	VMF_X68K	= VMF_X68030 | VMF_RXZ,
	VMF_LUNA	= VMF_LUNA1 | VMF_LUNA88K,
	VMF_M68K	= VMF_X68K | VMF_LUNA1,
	VMF_M88K	= VMF_LUNA88K,
	VMF_ALL		= 0xffffffff,
};

class VM
{
 public:
	VM();
	virtual ~VM();

	// 明示的な後始末
	void Dispose();

	// 動的なコンストラクション
	bool Create();

	// 初期化
	bool Init();

	// 設定の適用
	bool Apply();

	// 電源ボタンを押す。
	// XXX この辺はもうちょっと検討したほうがいい
	virtual void PowerButton() = 0;

	// 電源ボタンの状態を取得する。
	// モーメンタリスイッチの場合は常に false。
	virtual bool IsPowerButton() const { return false; }

	// VM の全デバイスの電源をオフ
	bool DevicePowerOff();

	// VM 電源オフ時のコールバックを設定する。(GUI から呼ばれる)
	void SetPowerOffCallback(void (*callback)());

	// 外部リセットボタンを押す。
	void ResetButton() { ResetHard(); }

	// ハードウェアリセット
	void ResetHard();

	// MPU の RESET 命令によるリセット
	void ResetSoft();

	// BusErr デバイスを取得する
	BusErrDevice *GetBusErrDevice() const { return buserr.get(); }

 protected:
	// VM の全デバイスの電源をオン
	bool DevicePowerOn();

	std::unique_ptr<BusErrDevice> buserr {};

	void (*poweroff_callback)() {};
};

extern std::unique_ptr<VM> gVM;
