//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxtextpanel.h"

//
// テキストスクリーンを表示するパネル
//
// wxPanel
//  ↓
// WXTextPanel  テキストフォントを管理するクラス
//  ↓
// WXTextScreen テキストスクリーンを表示するパネル
//
class WXTextScreen : public WXTextPanel
{
	using inherited = WXTextPanel;

 public:
	WXTextScreen(wxWindow *parent, IMonitor& monitor);
	virtual ~WXTextScreen() override;

	// 必要なら画面を更新して再描画する。
	// TextScreen が前回描画時から更新されていたら、このパネルを再描画する。
	void RefreshIfUpdated();

	// フォントサイズ変更
	void SetFontSize(FontId fontid) override;

	// 更新頻度設定
	void SetRate(int hz);

	// サイズ (桁数×行数) を取得する
	const wxSize GetSize() const { return wxSize(ts.GetCol(), ts.GetRow()); }

	// TextScreen のユーザデータ取得、設定
	uint64 GetUserData() const { return ts.userdata; }
	void SetUserData(uint64 value) { ts.userdata = value; }

 private:
	void OnPaint(wxPaintEvent&);
	void OnSize(wxSizeEvent&);
	void OnTimer(wxTimerEvent&);

	void Draw(wxDC& dc);

	std::unique_ptr<uint16[]> prevbuf {};
	std::unique_ptr<uint8[]> bmpbuf {};
	std::unique_ptr<wxImage> image {};
	std::unique_ptr<wxBitmap> bitmap {};

	IMonitor& monitor;
	TextScreen ts {};
	int view_width {};
	int view_height {};

	wxTimer timer {};

	wxDECLARE_EVENT_TABLE();
};
