//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

#pragma once

#include "wxsubwindow.h"
#include "object.h"

//
// LUNA ビットマップパネル
//
class WXLunaBitmapPanel : public wxScrolledWindow
{
	using inherited = wxScrolledWindow;
 public:
	WXLunaBitmapPanel(wxWindow *parent);
	virtual ~WXLunaBitmapPanel() override;

 private:
	void OnSize(wxSizeEvent& event);
	void OnPaint(wxPaintEvent& event);
	void OnScrollWin(wxScrollWinEvent& event);
	void Draw();

	const uint8 *src {};		// Bitmap デバイス内のバッファを指す
	std::unique_ptr<uint8[]> bmpbuf {};	// 裏バッファ

	int view_x {};				// ビューの表示開始 X 座標
	int view_y {};				// ビューの表示開始 Y 座標
	int view_width {};			// ビュー幅
	int view_height {};			// ビュー高さ

	// 縦は1ドットずつ処理出来るので特筆することはない。
	// 横は少なくともバイト(8ドット)単位に制限したほうが実装が楽。
	// 今はロングワード単位で描画しているため 32ドット単位としている。
	static const int virtual_width = 2048;	// 仮想画面幅 [pixel]
	static const int virtual_height = 1024;	// 仮想画面高さ [pixel]
	static const int width_per_unit = 32;	// 横スクロール単位 [pixel]
	static const int height_per_unit = 1;	// 縦スクロール単位 [pixel]
	static_assert(virtual_width  % width_per_unit  == 0, "");
	static_assert(virtual_height % height_per_unit == 0, "");

	// イベントテーブル
	wxDECLARE_EVENT_TABLE();
};

//
// LUNA ビットマップウィンドウ
//
class WXLunaBitmapWindow : public WXSubWindow
{
	using inherited = WXSubWindow;
 public:
	WXLunaBitmapWindow(wxWindow *parent, wxWindowID id, const wxString& name);
	virtual ~WXLunaBitmapWindow() override;

 private:
	// タイマーイベント
	void OnTimer(wxTimerEvent& event);

	// グラフィックパネル
	WXLunaBitmapPanel *panel {};

	// タイマー
	wxTimer timer {};

	// イベントテーブル
	wxDECLARE_EVENT_TABLE();
};
